#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 08/04/2000
#################################################################################
#
# Post - Module part of Webchat designed to display posts based on information
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2R 0K9
#
#################################################################################

package Webchat::Post;						# Package name

use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Dbm; 						# DBM Addition
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter functionality


#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# GetPosts
#
# Using basic information about the user will return a Statement handle (more 
# lightweight then an array) that will be used in the display the information
# gathered.

sub GetPosts							# Database, Statement reference, Parameters
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement = shift;					# Statement handle (reference)
	my $param = shift;					# Parameter list

	my $formatting;						# Formatting used	
	my $delay;						# Delay to retreive posts

	my $ignored;						# Ignored person
	
	my %realm;						# Portal HASH
	

	#####################
	# Program area


	# Determine time delay
	unless (defined($param->{'NUM'}))
	{
		Webchat::Dbm::DBMOpenRealm(\%realm);		# Link with hash
		$delay = (time - $realm{'SetTimeout'});		# Determin value
		Webchat::Dbm::DBMClose(\%realm);		# Unlink from hash
	}	


	# Format handling
	# Retreive formatting used
	($formatting) = Webchat::Database::DataGetFormatting($Database, $param->{'USER'});

	# Determin field to use
	if ($formatting eq 'complete')				# Full formatting
	{ $formatting = 'PostFullFormat'; }
	elsif ($formatting eq 'spartan')			# Spartan formatting
	{ $formatting = 'PostPartialFormat'; }
	elsif ($formatting eq 'imageless')			# Full less images
	{ $formatting = 'PostImagelessFormat'; }

	# List of ignored
	$ignored = IgnoreHandler($Database, $param);

	# Number based search
	if (defined($param->{'NUM'}))
	{
		# Initialize and execute
		${ $Statement } = $Database->prepare("SELECT PostIDNumber, $formatting FROM Post 
					WHERE PostIDNumber > ? AND RealmName=? AND (PostTo='all' OR PostTo=?) 
					$ignored ORDER BY PostIDNumber");
		${ $Statement }->execute($param->{'NUM'}, $param->{'ROOM'}, $param->{'USER'});
	}
	# Time based search
	else
	{
		# Initialize and execute
		${ $Statement } = $Database->prepare("SELECT PostIDNumber, $formatting FROM Post 
					WHERE PostTimeStamp > ? AND RealmName=? AND (PostTo='all' OR PostTo=?) 
					$ignored ORDER BY PostIDNumber");
		${ $Statement }->execute($delay, $param->{'ROOM'}, $param->{'USER'});
	}
}


#####################
# Handle Posts
#
# Will display all retreived posts to screen, a fairly straightforward 
# operation, will also give a value to NUM once it knows the last post
# received

sub HandlePosts							# Database, Parameters
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Statement handle (reference)
	my $param = shift;					# Parameter list

	my $post;						# Post itself
	my $postnum;						# Post number

	my %chat;						# Chat hash


	#####################
	# Program area

	# Link hash with DBM
	Webchat::Dbm::DBMOpenChat(\%chat);

	# Retreive posts
	GetPosts($Database, \$Statement, $param);	

	# Run though posts and print
	while ( ($_, $post) = $Statement->fetchrow() )
	{ 
		print "$chat{'TagSeperator'}";			# Seperator line
		print $post;					# Display post 
		$postnum = $_;					# Give value to postnum
	}

	# Finish query
	$Statement->finish();
	
	# Change value of NUM or define
	if (defined($postnum))
	{ $param->{'NUM'} = ($postnum - 1); }

	# Link hash with DBM
	Webchat::Dbm::DBMClose(\%chat);
}


#####################
# Handle Stream
#
# Will display all retreived posts to screen, a fairly straightforward 
# operation.  This unlike HandlePosts will not stop until the stream
# is ended

sub HandleStream
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Statement handle (reference)
	my $cgi = shift;					# CGI Handle
	my $param = shift;					# Parameter list

	my $post;						# Post itself
	my $postnum;						# Post number
	my $tmp;						# Temporary scalar holder
	my $size = 0;						# Size of posts

	my %chat;						# Chat hash


	#####################
	# Program area

	# Assign seperator
	Webchat::Dbm::DBMOpenChat(\%chat);
	my $seperator = $chat{'TagSeperator'};

	while ($size < $chat{'SetStreamSize'}) 
	{
		# Retreive posts
		GetPosts($Database, \$Statement, $param);	

		# Run though posts and print
		while ( ($tmp, $post) = $Statement->fetchrow() )
		{ 
			# Display
			print "$seperator";			# Seperator line
			print $post;				# Display post 
	
			# Assignment
			$postnum = $tmp;			# Give value to postnum
			$size += length($post);
		}

		# Finish query
		$Statement->finish();
	
		# Change value of NUM or define
		if (defined($postnum))
		{ $param->{'NUM'} = ($postnum); }

		sleep(5);					# Sleep before next pass
	}

	# Retreive URL
	my $url = $cgi->url();
	my $inline = Webchat::Param::EmbedInline($Database, $cgi, %{$param});

	# Reload link upon reaching bottom
	print "<P><CENTER><A HREF=\"$url$inline\&MAIN=aff\">$chat{'TagLnkReload'}</A></CENTER></P>";

	# Untie DBM from hash
	Webchat::Dbm::DBMClose(\%chat);
}


#####################
# Ignore Handler
#
# Will create a string to append to the end of the SQL statement, as
# to ignore the specified list of people.

sub IgnoreHandler						# Database, Parameters
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Statement handle (reference)
	my $param = shift;					# Parameter list

	my $string;						# Ignored string
	my $ignored;						# Ignored person
	my @ignored;						# Ignored group

	#####################
	# Program area

	# Retreive list of the damned
	Webchat::Database::GetListPuppetIgnored($Database, \@ignored, $param->{'USER'});

	# As long as there are entries
	while ($ignored = shift @ignored)
	{
		# If the string is defined
		if (defined($string))
		{ $string = "$string AND PuppetName <> '$ignored'"; }

		# Initialize for next pass
		else
		{ $string = " AND PuppetName <> '$ignored'"; }		
	}

	# Checks before returning
	if (defined($string))
	{ return $string;  }					# Return string

	else							# Empty string
	{ return ' '; } 
}


1;								# Return true
